import { Component, OnDestroy, OnInit } from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { Router } from '@angular/router';
import { Store } from '@ngrx/store';
import { Subscription } from 'rxjs';
import { AppState } from 'src/app/app.reducers';
import { MessageService } from 'src/app/Shared/services/message.service';
import {
  passwordPattern,
  repeatPasswordValidator,
} from 'src/app/Shared/validators/password.validator';
import * as AuthAction from '../../actions';

@Component({
  selector: 'app-change-password',
  templateUrl: './change-password.component.html',
  styleUrls: ['./change-password.component.scss'],
})
export class ChangePasswordComponent implements OnInit, OnDestroy {
  username: string;
  password: FormControl;
  oldpassword: FormControl;
  repeatpassword: FormControl;
  changeForm: FormGroup;
  subscription: Subscription | null;

  constructor(
    private formBuilder: FormBuilder,
    private store: Store<AppState>,
    private router: Router,
    private messageService: MessageService
  ) {
    this.username = '';
    this.oldpassword = new FormControl('', [Validators.required]);
    this.password = new FormControl('', [
      Validators.required,
      Validators.minLength(8),
      Validators.maxLength(16),
      Validators.pattern(passwordPattern),
    ]);
    this.repeatpassword = new FormControl('');

    this.changeForm = this.formBuilder.group(
      {
        password: this.password,
        oldpassword: this.oldpassword,
        repeatpassword: this.repeatpassword,
      },
      { validators: repeatPasswordValidator }
    );

    this.subscription = this.store.select('auth').subscribe((auth) => {
      if (auth.logged) {
        this.username = auth.authentication.username;
      }
      if (auth.passwordChanged) {
        this.router.navigateByUrl('home');
      }
      if (auth.error) {
        messageService.showHttpResponseError(
          'loginFeedback',
          auth.error.payload
        );
      }
    });
  }

  ngOnInit(): void {}

  ngOnDestroy() {
    if (this.subscription) this.subscription.unsubscribe();
  }

  changePassword(): void {
    if (!this.changeForm.invalid) {
      this.store.dispatch(
        AuthAction.changePassword({
          credentials: {
            password: this.oldpassword.value,
            newpassword: this.password.value,
            username: this.username,
          },
        })
      );
    }
  }
}
