import { HttpErrorResponse } from '@angular/common/http';
import { Component, OnInit } from '@angular/core';
import {
  AbstractControl,
  FormBuilder,
  FormControl,
  FormGroup,
  ValidationErrors,
  ValidatorFn,
  Validators,
} from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { MessageService } from 'src/app/Shared/services/message.service';
import { ArticleDTO } from '../../model/article.dto';
import { ArticleService } from '../../services/ordenances.service';

@Component({
  selector: 'app-ordenances-form',
  templateUrl: './ordenances-form.component.html',
  styleUrls: ['./ordenances-form.component.scss'],
})
export class OrdenancesFormComponent implements OnInit {
  article: ArticleDTO;
  codi: FormControl;
  descripcio: FormControl;
  import_sancio: FormControl;
  import_minim: FormControl;
  import_maxim: FormControl;
  articleForm: FormGroup;
  karticle: string | null;
  accio: string | null;
  titulo: string;
  formSubmited: boolean;

  constructor(
    private formBuilder: FormBuilder,
    private router: Router,
    private activatedRoute: ActivatedRoute,
    private messageService: MessageService,
    private articleService: ArticleService
  ) {
    this.formSubmited = false;
    this.accio = this.activatedRoute.snapshot.paramMap.get('accio');
    this.karticle = this.activatedRoute.snapshot.paramMap.get('karticle');
    this.article = new ArticleDTO(0, '', '', null, null, null);
    this.codi = new FormControl('', [
      Validators.required,
      Validators.maxLength(10),
    ]);
    this.descripcio = new FormControl('', [
      Validators.required,
      Validators.minLength(10),
      Validators.maxLength(50),
    ]);
    this.import_sancio = new FormControl();
    this.import_minim = new FormControl();
    this.import_maxim = new FormControl();

    this.titulo = 'Alta';

    this.articleForm = this.formBuilder.group(
      {
        codi: this.codi,
        descripcio: this.descripcio,
        importSancio: this.import_sancio,
        importMinim: this.import_minim,
        importMaxim: this.import_maxim,
      },
      { validators: this.validacioImports }
    );
  }

  ngOnInit(): void {
    if (this.karticle) {
      if (this.accio === 'update') this.titulo = 'Modificación';
      if (this.accio === 'delete') this.titulo = 'Borrado';
      //this.passwordValidators();
      this.articleService.getArticle(this.karticle).subscribe(
        (article: ArticleDTO) => {
          this.article = article;
          this.codi.setValue(this.article.codi);
          this.descripcio.setValue(this.article.descripcio);
          this.changeNumberValue(this.import_sancio, this.article.importSancio);
          this.changeNumberValue(this.import_minim, this.article.importMinim);
          this.changeNumberValue(this.import_maxim, this.article.importMaxim);
        },
        (error: HttpErrorResponse) =>
          this.messageService.showHttpResponseError('articleFeedback', error)
      );
    }
  }

  private changeNumberValue(control: FormControl, value: number | null) {
    control.setValue(value ? value : null);
  }

  private updateArticle(): void {
    if (this.karticle) {
      this.article.karticle = Number(this.karticle);
      this.articleService.saveArticle(this.article).subscribe(
        () => {
          this.router.navigateByUrl('ordenances/ordenances-list');
        },
        (error: HttpErrorResponse) => {
          this.messageService.showHttpResponseError('articleFeedback', error);
        }
      );
    }
  }

  private deleteArticle(): void {
    if (this.karticle) {
      this.article.karticle = Number(this.karticle);
      this.articleService.deleteArticle(this.article).subscribe(
        () => {
          this.router.navigateByUrl('ordenances/ordenances-list');
        },
        (error: HttpErrorResponse) => {
          this.messageService.showHttpResponseError('articleFeedback', error);
        }
      );
    }
  }
  private createArticle(): void {
    this.articleService.addArticle(this.article).subscribe(
      () => {
        this.router.navigateByUrl('ordenances/ordenances-list');
      },
      (error: HttpErrorResponse) => {
        this.messageService.showHttpResponseError(
          'articleFeedback',
          error
        );
      }
    );
  }

  saveArticle(): void {
    this.formSubmited = true;
    if (this.articleForm.invalid) {
      return;
    }
    this.article = this.articleForm.value;

    if (this.accio) {
      if (this.accio === 'insert') this.createArticle();
      else {
        if (this.accio === 'update') this.updateArticle();
        else this.deleteArticle();
      }
    }
  }

  exit(): void {
    this.router.navigateByUrl('ordenances/ordenances-list');
  }

  private validacioImports: ValidatorFn = (
    control: AbstractControl
  ): ValidationErrors | null => {
    const import_sancio = this.getControlValue(control, 'importSancio');
    const import_minim = this.getControlValue(control, 'importMinim');
    const import_maxim = this.getControlValue(control, 'importMaxim');

    if (this.checkImportPendent(import_sancio, import_minim, import_maxim))
      return { importPendent: true };
    if (this.checkImportDuplicat(import_sancio, import_minim, import_maxim))
      return { importDuplicat: true };
    if (this.checkRangIncomplet(import_minim, import_maxim))
      return { importIncomplet: true };
    if (this.checkMinMax(import_minim, import_maxim))
      return { importMinMax: true };
    return null;
  };

  private checkImportPendent(
    import_sancio: string,
    import_minim: string,
    import_maxim: string
  ): boolean {
    return import_sancio === '' && import_minim === '' && import_maxim === '';
  }

  private checkImportDuplicat(
    import_sancio: string,
    import_minim: string,
    import_maxim: string
  ): boolean {
    return import_sancio !== '' && (import_minim !== '' || import_maxim !== '');
  }
  private checkRangIncomplet(
    import_minim: string,
    import_maxim: string
  ): boolean {
    return (
      (import_minim === '' && import_maxim !== '') ||
      (import_minim !== '' && import_maxim === '')
    );
  }

  private checkMinMax(import_minim: string, import_maxim: string): boolean {
    return (
      import_minim !== '' &&
      import_maxim !== '' &&
      Number(import_minim) >= Number(import_maxim)
    );
  }

  private getControlValue(control: AbstractControl, fieldname: string): string {
    const field = control.get(fieldname);
    return field === undefined || field === null || field.value === null
      ? ''
      : field.value;
  }
}
