import { BreakpointObserver, Breakpoints } from '@angular/cdk/layout';
import { Component, OnDestroy, OnInit } from '@angular/core';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { Router } from '@angular/router';
import { Store } from '@ngrx/store';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { AppState } from 'src/app/app.reducers';
import { checkAllowedAction } from 'src/app/Shared/model/security-manager';
import { MessageService } from 'src/app/Shared/services/message.service';
import { ArticleDTO } from '../../model/article.dto';
import { ArticleService } from '../../services/ordenances.service';

@Component({
  selector: 'app-ordenances-list',
  templateUrl: './ordenances-list.component.html',
  styleUrls: ['./ordenances-list.component.scss'],
})
export class OrdenancesListComponent implements OnInit, OnDestroy {
  articles: ArticleDTO[];
  filterForm: FormGroup;
  descripcio: FormControl;
  codi: FormControl;
  articleDisplayedColumns: string[] = [
    'codi',
    'descripcio',
    'impsancio',
    'impminim',
    'impmaxim',
    'actions',
  ];
  canSelect: boolean;
  canUpdate: boolean;
  canInsert: boolean;
  canDelete: boolean;
  destroyed = new Subject<void>();
  isMobile: boolean;

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private articleService: ArticleService,
    private store: Store<AppState>,
    private messageService: MessageService,
    private breakpointObserver: BreakpointObserver
  ) {
    this.isMobile = false;
    this.articles = [];
    this.descripcio = new FormControl('');
    this.codi = new FormControl('');
    this.filterForm = this.formBuilder.group({
      descripcio: this.descripcio,
      codi: this.codi,
    });
    this.canSelect = false;
    this.canUpdate = false;
    this.canInsert = false;
    this.canDelete = false;
    this.initObserver();
    this.store.select('auth').subscribe((auth) => {
      if (auth.authentication.access_token) {
        this.canSelect = checkAllowedAction(
          auth.authentication,
          'ORDENANCES',
          'SELECT'
        );
        this.canUpdate = checkAllowedAction(
          auth.authentication,
          'ORDENANCES',
          'UPDATE'
        );
        this.canInsert = checkAllowedAction(
          auth.authentication,
          'ORDENANCES',
          'INSERT'
        );
        this.canDelete = checkAllowedAction(
          auth.authentication,
          'ORDENANCES',
          'DELETE'
        );
      }
    });
  }

  ngOnInit(): void {}

  filterByName(): void {
    if (this.descripcio.value !== '') {
      this.articleService.filterByName(this.descripcio.value).subscribe(
        (articles: ArticleDTO[]) => {
          this.articles = articles;
        },
        (error) => {
          this.messageService.showHttpResponseError(
            'ordenancesFeedback',
            error
          );
        }
      );
    }
  }

  filterByCode(): void {
    if (this.codi.value !== '') {
      this.articleService.filterByCode(this.codi.value).subscribe(
        (articles: ArticleDTO[]) => {
          this.articles = articles;
        },
        (error) => {
          this.messageService.showHttpResponseError(
            'ordenancesFeedback',
            error
          );
        }
      );
    }
  }

  hasData(): boolean {
    return this.articles.length > 0;
  }

  afegirArticle(): void {
    this.router.navigateByUrl('ordenances/ordenances-form/insert/');
  }

  updateArticle(article: ArticleDTO): void {
    if (article.karticle)
      this.router.navigateByUrl(
        'ordenances/ordenances-form/update/' + article.karticle
      );
  }

  deleteArticle(article: ArticleDTO): void {
    if (article.karticle)
      this.router.navigateByUrl(
        'ordenances/ordenances-form/delete/' + article.karticle
      );
  }

  private initObserver() {
    this.breakpointObserver
      .observe([
        Breakpoints.XSmall,
        Breakpoints.Small,
        Breakpoints.Medium,
        Breakpoints.Large,
        Breakpoints.XLarge,
      ])
      .pipe(takeUntil(this.destroyed))
      .subscribe((result) => {
        for (const query of Object.keys(result.breakpoints)) {
          if (result.breakpoints[query]) {
            this.isMobile = query === Breakpoints.XSmall;
          }
        }
      });
  }

  ngOnDestroy() {
    this.destroyed.next();
    this.destroyed.complete();
  }
}
