import { BreakpointObserver, Breakpoints } from '@angular/cdk/layout';
import { Component, OnDestroy, OnInit } from '@angular/core';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import { Router } from '@angular/router';
import { Store } from '@ngrx/store';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { AppState } from 'src/app/app.reducers';
import { checkAllowedAction } from 'src/app/Shared/model/security-manager';
import { MessageService } from 'src/app/Shared/services/message.service';
import { getTpDocument } from 'src/app/Shared/validators/document.validator';
import { ActaDTO } from '../../model/acta.dto';
import { MenuSancioService } from '../../services/menu-sancions.service';
import { SancioService } from '../../services/sancio.service';

@Component({
  selector: 'app-actes-list',
  templateUrl: './actes-list.component.html',
  styleUrls: ['./actes-list.component.scss'],
})
export class ActesListComponent implements OnInit, OnDestroy {
  actes: ActaDTO[];
  filterForm: FormGroup;
  document: FormControl;
  year: FormControl;
  numActa: FormControl;
  actaDisplayedColumns: string[] = [
    'data',
    'numacta',
    'document',
    'passaport',
    'nom',
    'estat',
    'actions',
  ];
  canSelect: boolean;
  canUpdate: boolean;
  canInsert: boolean;
  canDelete: boolean;
  destroyed = new Subject<void>();
  isMobile: boolean;

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private sancioService: SancioService,
    private messageService: MessageService,
    private breakpointObserver: BreakpointObserver,
    private store: Store<AppState>,
    private menuSancioService: MenuSancioService
  ) {
    this.isMobile = false;
    this.actes = [];
    this.document = new FormControl('');
    this.year = new FormControl('');
    this.numActa = new FormControl('');
    this.filterForm = this.formBuilder.group({
      document: this.document,
      year: this.year,
      numActa: this.numActa,
    });
    this.canSelect = false;
    this.canUpdate = false;
    this.canInsert = false;
    this.canDelete = false;
    this.initObserver();
    this.store.select('auth').subscribe((auth) => {
      if (auth.authentication.access_token) {
        this.canSelect = checkAllowedAction(
          auth.authentication,
          'SANCIO',
          'SELECT'
        );
        this.canUpdate = checkAllowedAction(
          auth.authentication,
          'SANCIO',
          'UPDATE'
        );
        this.canInsert = checkAllowedAction(
          auth.authentication,
          'SANCIO',
          'INSERT'
        );
      }
    });
  }

  ngOnInit(): void {}

  filterByDocument(): void {
    if (this.document.value !== '') {
      this.sancioService.filterByDocument(this.document.value).subscribe(
        (actes: ActaDTO[]) => {
          this.actes = actes;
        },
        (error) => {
          this.messageService.showHttpResponseError('sancionsFeedback', error);
        }
      );
    }
  }

  filterByNumActa(): void {
    if (this.year.value !== '' && this.numActa.value !== '') {
      this.sancioService
        .filterByNumActa(this.year.value, this.numActa.value)
        .subscribe(
          (actes: ActaDTO[]) => {
            this.actes = actes;
          },
          (error) => {
            this.messageService.showHttpResponseError(
              'sancionsFeedback',
              error
            );
          }
        );
    }
  }

  hasData(): boolean {
    return this.actes.length > 0;
  }

  afegirActa(): void {
    this.menuSancioService.setActa(new ActaDTO());
    this.router.navigateByUrl('sancio/menu');
  }

  updateActa(acta: ActaDTO): void {
    this.menuSancioService.setActa(acta);
    this.menuSancioService.setUrl('sancio/actes-list');
    this.router.navigateByUrl('sancio/menu');
  }

  private initObserver() {
    this.breakpointObserver
      .observe([
        Breakpoints.XSmall,
        Breakpoints.Small,
        Breakpoints.Medium,
        Breakpoints.Large,
        Breakpoints.XLarge,
      ])
      .pipe(takeUntil(this.destroyed))
      .subscribe((result) => {
        for (const query of Object.keys(result.breakpoints)) {
          if (result.breakpoints[query]) {
            this.isMobile = query === Breakpoints.XSmall;
          }
        }
      });
  }

  getNom(acta: ActaDTO): string {
    let denunciat = acta.denunciat;
    const tpdocu = getTpDocument(denunciat.document);
    if (tpdocu === 'CIF') return denunciat.dencomerc;
    else
      return (
        denunciat.nom + ' ' + denunciat.llinatge1 + ' ' + denunciat.llinatge2
      );
  }

  getEstat(acta: ActaDTO) {
    const estat = acta.rebut?.estat;
    return !estat ? '' : estat === 'P' ? 'PENDIENTE' : 'COBRADO';
  }

  ngOnDestroy() {
    this.destroyed.next();
    this.destroyed.complete();
  }
}
