import { HttpErrorResponse } from '@angular/common/http';
import { Component, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { FormControl } from '@angular/forms';
import { Store } from '@ngrx/store';
import { Subscription } from 'rxjs';
import { AppState } from 'src/app/app.reducers';
import { MessageService } from 'src/app/Shared/services/message.service';
import { DrawingDirective } from '../../directives/drawing.directive';
import { ActaDTO } from '../../model/acta.dto';
import { FirmaActaDTO } from '../../model/firma-acta.dto';
import { MenuSancioService } from '../../services/menu-sancions.service';
import { SancioService } from '../../services/sancio.service';

@Component({
  selector: 'app-firma',
  templateUrl: './firma.component.html',
  styleUrls: ['./firma.component.scss'],
})
export class FirmaComponent implements OnInit, OnDestroy {
  @ViewChild('canvas1') dcanvas1?: DrawingDirective;
  @ViewChild('canvas2') dcanvas2?: DrawingDirective;
  @ViewChild('canvas3') dcanvas3?: DrawingDirective;
  acta: ActaDTO;
  username: string;
  agent2: FormControl;
  agente1_signed: boolean;
  agente2_signed: boolean;
  denunciat_signed: boolean;

  subscription: Subscription;

  constructor(
    private menuSancioService: MenuSancioService,
    private sancioService: SancioService,
    private store: Store<AppState>,
    private messageService: MessageService
  ) {
    this.agente1_signed = true;
    this.agente2_signed = true;
    this.denunciat_signed = true;

    this.acta = menuSancioService.getActa();
    this.subscription = menuSancioService.change.subscribe(() => {
      menuSancioService.validated.emit(
        this.acta.kacta ? true : this.isValidForm()
      );
    });
    this.username = '';
    this.agent2 = new FormControl('');
    this.store.select('auth').subscribe((auth) => {
      this.username = auth.authentication.username;
    });
  }

  //Para acceder a info de la directiva
  //https://stackoverflow.com/questions/36345618/get-reference-to-a-directive-used-in-a-component
  ngOnInit(): void {}

  ngAfterViewInit() {
    if (this.acta.kacta) {
      if (this.dcanvas1) this.dcanvas1.disabled = true;
      if (this.dcanvas2) this.dcanvas2.disabled = true;
      if (this.dcanvas3) this.dcanvas3.disabled = true;

      if (!this.acta.firmes || this.acta.firmes.length === 0) {
        this.sancioService.getFirmes(this.acta.kacta.toString()).subscribe(
          (firmes) => {
            this.acta.firmes = [...firmes];
            this.recuperaFirmes();
          },
          (error: HttpErrorResponse) => {
            this.messageService.showHttpResponseError('firmesFeedback', error);
          }
        );
      } else this.recuperaFirmes();
    } else this.recuperaFirmes();
  }

  cleanData1() {
    this.dcanvas1?.clean();
  }

  cleanData2() {
    this.dcanvas2?.clean();
  }

  cleanData3() {
    this.dcanvas3?.clean();
  }

  public isValidForm(): boolean {
    let isValidForm: boolean = this.checkFirmes();
    if (isValidForm) {
      let firmes: FirmaActaDTO[] = this.buildFirmes();
      this.acta = {
        ...this.acta,
        firmes: firmes,
      };
      this.menuSancioService.setActa(this.acta);
      return true;
    } else return false;
  }

  private buildFirmes(): FirmaActaDTO[] {
    let firmes: FirmaActaDTO[] = [];
    if (this.dcanvas1) {
      firmes.push(
        new FirmaActaDTO(this.dcanvas1.getDataURL(), this.username, 'A1')
      );
    }
    if (this.agent2.value !== '' && this.dcanvas2) {
      firmes.push(
        new FirmaActaDTO(this.dcanvas2.getDataURL(), this.agent2.value, 'A2')
      );
    }
    if (this.dcanvas3) {
      firmes.push(
        new FirmaActaDTO(
          this.dcanvas3.getDataURL(),
          this.acta.denunciat.document !== ''
            ? this.acta.denunciat.document
            : this.acta.denunciat.passaport,
          'D1'
        )
      );
    }
    return firmes;
  }

  private checkFirmes() {
    if (this.acta.kacta) return true;
    this.agente1_signed = !this.dcanvas1 || this.dcanvas1.dirty;
    this.agente2_signed =
      this.agent2.value === '' || !this.dcanvas2 || this.dcanvas2.dirty;
    this.denunciat_signed = !this.dcanvas3 || this.dcanvas3.dirty;
    return this.agente1_signed && this.agente2_signed && this.denunciat_signed;
  }

  private recuperaFirmes(): void {
    if (this.acta.firmes) {
      this.acta.firmes.forEach((firma: FirmaActaDTO) => {
        if (firma.origen === 'A1' && this.dcanvas1) {
          this.agente1_signed = true;
          this.dcanvas1.paintCanvas(firma.data);
        }
        if (firma.origen === 'A2' && this.dcanvas2) {
          this.agente2_signed = true;
          this.dcanvas2.paintCanvas(firma.data);
          this.agent2.setValue(firma.firmant);
        }
        if (firma.origen === 'D1' && this.dcanvas3) {
          this.denunciat_signed = true;
          this.dcanvas3.paintCanvas(firma.data);
        }
      });
    }
  }

  ngOnDestroy(): void {
    this.subscription.unsubscribe();
  }

  isNewActa(): boolean {
    return !this.acta.kacta;
  }
}
