import { BreakpointObserver, Breakpoints } from '@angular/cdk/layout';
import { HttpErrorResponse } from '@angular/common/http';
import { Component, OnDestroy, OnInit } from '@angular/core';
import { WebcamImage, WebcamInitError, WebcamUtil } from 'ngx-webcam';
import { Observable, Subject, Subscription } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { MessageService } from 'src/app/Shared/services/message.service';
import { ActaDTO } from '../../model/acta.dto';
import { FotoActaDTO } from '../../model/foto-acta.dto';
import { MenuSancioService } from '../../services/menu-sancions.service';
import { SancioService } from '../../services/sancio.service';

@Component({
  selector: 'app-fotos',
  templateUrl: './fotos.component.html',
  styleUrls: ['./fotos.component.scss'],
})
export class FotosComponent implements OnInit, OnDestroy {
  acta: ActaDTO;
  allowCameraSwitch = true;
  multipleWebcamsAvailable = false;
  deviceId: string = '';
  facingMode: string = 'environment';
  messages: any[] = [];
  destroyed = new Subject<void>();
  camera_width: number = 800;
  camera_height: number = 600;
  subscription: Subscription;

  // latest snapshot
  public webcamImages: FotoActaDTO[] = [];

  // webcam snapshot trigger
  private trigger: Subject<void> = new Subject<void>();
  // switch to next / previous / specific webcam; true/false: forward/backwards, string: deviceId
  private nextWebcam: Subject<boolean | string> = new Subject<
    boolean | string
  >();

  constructor(
    private breakpointObserver: BreakpointObserver,
    private menuSancioService: MenuSancioService,
    private sancioService: SancioService,
    private messageService: MessageService
  ) {
    this.initObserver();
    this.acta = menuSancioService.getActa();
    if (this.acta.fotos) this.webcamImages = this.acta.fotos;
    this.subscription = menuSancioService.change.subscribe(() => {
      menuSancioService.validated.emit(this.buildFotos());
    });
  }

  public ngOnInit(): void {
    this.readAvailableVideoInputs();
  }

  ngAfterViewInit() {
    if (this.acta.kacta) {
      if (!this.acta.fotos || this.acta.fotos.length === 0) {
        this.sancioService.getFotos(this.acta.kacta.toString()).subscribe(
          (fotos) => {
            this.acta.fotos = [...fotos];
            this.webcamImages = [...fotos];
          },
          (error: HttpErrorResponse) => {
            this.messageService.showHttpResponseError('firmesFeedback', error);
          }
        );
      }
    }
  }

  public triggerSnapshot(): void {
    this.trigger.next();
  }

  public handleInitError(error: WebcamInitError): void {
    this.messages.push(error);
    if (
      error.mediaStreamError &&
      error.mediaStreamError.name === 'NotAllowedError'
    ) {
      this.addMessage('User denied camera access');
    }
  }

  public showNextWebcam(directionOrDeviceId: boolean | string): void {
    // true => move forward through devices
    // false => move backwards through devices
    // string => move to device with given deviceId
    this.nextWebcam.next(directionOrDeviceId);
  }

  public handleImage(webcamImage: WebcamImage): void {
    this.addMessage('Received webcam image');
    this.webcamImages.push(new FotoActaDTO(webcamImage.imageAsDataUrl));
  }

  public cameraWasSwitched(deviceId: string): void {
    this.addMessage('Active device: ' + deviceId);
    this.deviceId = deviceId;
    this.readAvailableVideoInputs();
  }

  addMessage(message: any): void {
    this.messages.unshift(message);
  }

  public get triggerObservable(): Observable<void> {
    return this.trigger.asObservable();
  }

  public get nextWebcamObservable(): Observable<boolean | string> {
    return this.nextWebcam.asObservable();
  }

  public get videoOptions(): MediaTrackConstraints {
    const result: MediaTrackConstraints = {};
    if (this.facingMode && this.facingMode !== '') {
      result.facingMode = { ideal: this.facingMode };
    }

    return result;
  }

  delete(index: number): void {
    this.webcamImages = this.webcamImages.filter((webcamImage, i) => {
      return index !== i;
    });
  }

  private readAvailableVideoInputs() {
    WebcamUtil.getAvailableVideoInputs().then(
      (mediaDevices: MediaDeviceInfo[]) => {
        this.multipleWebcamsAvailable = mediaDevices && mediaDevices.length > 1;
      }
    );
  }

  private buildFotos(): boolean {
    if (this.isNewActa()) {
      this.acta.fotos = this.webcamImages;
      this.menuSancioService.setActa(this.acta);
    }

    return true;
  }

  private initObserver() {
    this.breakpointObserver
      .observe([
        Breakpoints.XSmall,
        Breakpoints.Small,
        Breakpoints.Medium,
        Breakpoints.Large,
        Breakpoints.XLarge,
      ])
      .pipe(takeUntil(this.destroyed))
      .subscribe((result) => {
        for (const query of Object.keys(result.breakpoints)) {
          if (result.breakpoints[query]) {
            if (query === Breakpoints.Small || query === Breakpoints.XSmall) {
              this.camera_width = 375;
              this.camera_height = 280;
            } else {
              this.camera_width = 800;
              this.camera_height = 600;
            }
          }
        }
      });
  }

  isNewActa(): boolean {
    return !this.acta.kacta;
  }
  ngOnDestroy() {
    this.destroyed.next();
    this.destroyed.complete();
    this.subscription.unsubscribe();
  }
}
