import { BreakpointObserver, Breakpoints } from '@angular/cdk/layout';
import { Location } from '@angular/common';
import { HttpErrorResponse } from '@angular/common/http';
import { Component, OnDestroy, OnInit } from '@angular/core';
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { Store } from '@ngrx/store';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { AppState } from 'src/app/app.reducers';
import { checkAllowedAction } from 'src/app/Shared/model/security-manager';
import { MessageService } from 'src/app/Shared/services/message.service';
import { getTpDocument } from 'src/app/Shared/validators/document.validator';
import { ActaDTO } from '../../model/acta.dto';
import { IdentificacioDTO } from '../../model/identificacio.dto';
import { MenuSancioService } from '../../services/menu-sancions.service';
import { PersonaService } from '../../services/persona.service';
import { SancioService } from '../../services/sancio.service';

@Component({
  selector: 'app-persona-info',
  templateUrl: './persona-info.component.html',
  styleUrls: ['./persona-info.component.scss'],
})
export class PersonaInfoComponent implements OnInit, OnDestroy {
  kpersona: number;
  persona: IdentificacioDTO;
  actes: ActaDTO[];
  actaDisplayedColumns: string[] = [
    'data',
    'numacta',
    'document',
    'passaport',
    'nom',
    'estat',
    'actions',
  ];
  canSelect: boolean;
  canUpdate: boolean;
  canInsert: boolean;
  canDelete: boolean;
  destroyed = new Subject<void>();
  isMobile: boolean;

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private sancioService: SancioService,
    private personaService: PersonaService,
    private messageService: MessageService,
    private breakpointObserver: BreakpointObserver,
    private store: Store<AppState>,
    private menuSancioService: MenuSancioService,
    private activatedRoute: ActivatedRoute,
    private location: Location
  ) {
    this.isMobile = false;
    this.kpersona = Number(
      this.activatedRoute.snapshot.paramMap.get('kpersona')
    );
    this.persona = new IdentificacioDTO();
    this.actes = [];
    this.canSelect = false;
    this.canUpdate = false;
    this.canInsert = false;
    this.canDelete = false;
    this.initObserver();
    this.store.select('auth').subscribe((auth) => {
      if (auth.authentication.access_token) {
        this.canSelect = checkAllowedAction(
          auth.authentication,
          'SANCIO',
          'SELECT'
        );
        this.canUpdate = checkAllowedAction(
          auth.authentication,
          'SANCIO',
          'UPDATE'
        );
        this.canInsert = checkAllowedAction(
          auth.authentication,
          'SANCIO',
          'INSERT'
        );
      }
    });
  }

  ngOnInit(): void {
    this.personaService.findById(this.kpersona).subscribe(
      (persona: IdentificacioDTO) => {
        this.persona = persona;
      },
      (error: HttpErrorResponse) => {
        this.messageService.showHttpResponseError('infoFeedback', error);
      }
    );
    this.sancioService.filterByKPersona(this.kpersona).subscribe(
      (actes: ActaDTO[]) => {
        this.actes = actes;
      },
      (error: HttpErrorResponse) => {
        this.messageService.showHttpResponseError('infoFeedback', error);
      }
    );
  }

  hasActes(): boolean {
    return this.actes.length > 0;
  }

  afegirActa(): void {
    let newActa: ActaDTO = new ActaDTO();
    newActa.denunciat = this.persona;
    this.menuSancioService.setActa(newActa);
    this.menuSancioService.setActa(newActa);
    this.menuSancioService.setUrl('sancio/persones-list');
    this.router.navigateByUrl('sancio/menu');
  }

  updateActa(acta: ActaDTO): void {
    this.menuSancioService.setActa(acta);
    this.router.navigateByUrl('sancio/menu');
  }

  private initObserver() {
    this.breakpointObserver
      .observe([
        Breakpoints.XSmall,
        Breakpoints.Small,
        Breakpoints.Medium,
        Breakpoints.Large,
        Breakpoints.XLarge,
      ])
      .pipe(takeUntil(this.destroyed))
      .subscribe((result) => {
        for (const query of Object.keys(result.breakpoints)) {
          if (result.breakpoints[query]) {
            this.isMobile = query === Breakpoints.XSmall;
          }
        }
      });
  }

  getNom(acta: ActaDTO): string {
    let denunciat = acta.denunciat;
    const tpdocu = getTpDocument(denunciat.document);
    if (tpdocu === 'CIF') return denunciat.dencomerc;
    else
      return (
        denunciat.nom + ' ' + denunciat.llinatge1 + ' ' + denunciat.llinatge2
      );
  }

  getDataNaixement(): string {
    if (this.persona.dataneixement) {
      let strd = this.persona.dataneixement.toString();
      return (
        strd.substring(8, 10) +
        '/' +
        strd.substring(5, 7) +
        '/' +
        strd.substring(0, 4)
      );
    } else return '';
  }

  getNomComplet(): string {
    return (
      this.persona.nom +
      ' ' +
      this.persona.llinatge1 +
      ' ' +
      this.persona.llinatge2
    );
  }

  getEstat(acta: ActaDTO) {
    const estat = acta.rebut?.estat;
    return !estat ? '' : estat === 'P' ? 'PENDIENTE' : 'COBRADO';
  }

  ngOnDestroy() {
    this.destroyed.next();
    this.destroyed.complete();
  }

  back(): void {
    this.location.back();
  }
}
