import { BreakpointObserver, Breakpoints } from '@angular/cdk/layout';
import { Component, OnDestroy, OnInit } from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { Router } from '@angular/router';
import { Store } from '@ngrx/store';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { AppState } from 'src/app/app.reducers';
import { checkAllowedAction } from 'src/app/Shared/model/security-manager';
import { MessageService } from 'src/app/Shared/services/message.service';
import { getTpDocument } from 'src/app/Shared/validators/document.validator';
import { ActaDTO } from '../../model/acta.dto';
import { IdentificacioDTO } from '../../model/identificacio.dto';
import { SancionatDTO } from '../../model/sancionat.dto';
import { MenuSancioService } from '../../services/menu-sancions.service';
import { PersonaService } from '../../services/persona.service';

@Component({
  selector: 'app-persones-list',
  templateUrl: './persones-list.component.html',
  styleUrls: ['./persones-list.component.scss'],
})
export class PersonesListComponent implements OnInit, OnDestroy {
  sancionats: SancionatDTO[];
  filterForm: FormGroup;
  document: FormControl;
  nom: FormControl;
  sancionatDisplayedColumns: string[] = [
    'document',
    'passaport',
    'nom',
    'numsancions',
    'actions',
  ];
  canSelect: boolean;
  canInsert: boolean;
  destroyed = new Subject<void>();
  isMobile: boolean;

  constructor(
    private router: Router,
    private formBuilder: FormBuilder,
    private personaService: PersonaService,
    private messageService: MessageService,
    private breakpointObserver: BreakpointObserver,
    private store: Store<AppState>,
    private menuSancioService: MenuSancioService
  ) {
    this.isMobile = false;
    this.sancionats = [];
    this.document = new FormControl('');
    this.nom = new FormControl('', Validators.minLength(3));
    this.filterForm = this.formBuilder.group({
      document: this.document,
      nom: this.nom,
    });
    this.canSelect = false;
    this.canInsert = false;
    this.initObserver();
    this.store.select('auth').subscribe((auth) => {
      if (auth.authentication.access_token) {
        this.canSelect = checkAllowedAction(
          auth.authentication,
          'INFO',
          'SELECT'
        );
        this.canInsert = checkAllowedAction(
          auth.authentication,
          'INFO',
          'INSERT'
        );
      }
    });
  }

  ngOnInit(): void {}

  filterByDocument(): void {
    if (this.document.value !== '') {
      this.personaService
        .findSancionatByDocument(this.document.value)
        .subscribe(
          (sancionat: SancionatDTO) => {
            if (sancionat) this.sancionats = [sancionat];
            else this.sancionats = [];
          },
          (error) => {
            this.messageService.showHttpResponseError('infoFeedback', error);
          }
        );
    }
  }

  filterByNom(): void {
    if (this.nom.value !== '' && this.nom.value.length >= 3) {
      this.personaService.findSancionatByNom(this.nom.value).subscribe(
        (sancionats: SancionatDTO[]) => {
          this.sancionats = sancionats;
        },
        (error) => {
          this.messageService.showHttpResponseError('infoFeedback', error);
        }
      );
    }
  }

  hasData(): boolean {
    return this.sancionats.length > 0;
  }

  afegirActa(sancionat: SancionatDTO): void {
    this.personaService
      .findById(sancionat.kpersona)
      .subscribe((persona: IdentificacioDTO) => {
        let newActa: ActaDTO = new ActaDTO();
        newActa.denunciat = persona;
        this.menuSancioService.setActa(newActa);
        this.menuSancioService.setUrl('sancio/persones-list');
        this.router.navigateByUrl('sancio/menu');
      });
  }

  showInfo(sancionat: SancionatDTO): void {
    this.router.navigateByUrl('sancio/persona-info/' + sancionat.kpersona);
  }
  private initObserver() {
    this.breakpointObserver
      .observe([
        Breakpoints.XSmall,
        Breakpoints.Small,
        Breakpoints.Medium,
        Breakpoints.Large,
        Breakpoints.XLarge,
      ])
      .pipe(takeUntil(this.destroyed))
      .subscribe((result) => {
        for (const query of Object.keys(result.breakpoints)) {
          if (result.breakpoints[query]) {
            this.isMobile = query === Breakpoints.XSmall;
          }
        }
      });
  }

  getNom(sancionat: SancionatDTO): string {
    const tpdocu = getTpDocument(sancionat.document);
    if (tpdocu === 'CIF') return sancionat.dencomerc;
    else
      return (
        sancionat.nom + ' ' + sancionat.llinatge1 + ' ' + sancionat.llinatge2
      );
  }

  ngOnDestroy() {
    this.destroyed.next();
    this.destroyed.complete();
  }
}
