import { Directive, ElementRef, HostListener } from '@angular/core';

@Directive({
  selector: '[appDrawing]',
  exportAs: 'drawingdirective',
})
export class DrawingDirective {
  canvas: HTMLCanvasElement;
  context: CanvasRenderingContext2D | null;
  drawing: boolean;
  lastX: number;
  lastY: number;
  dirty: boolean;
  disabled: boolean;

  constructor(private el: ElementRef) {
    this.disabled = false;
    this.drawing = false;
    this.lastX = 0;
    this.lastY = 0;
    this.canvas = el.nativeElement;
    this.context = this.canvas.getContext('2d');
    this.dirty = false;
  }

  startDrawing(event: any) {
    event.preventDefault();
    if (this.disabled) return;
    this.dirty = true;
    this.disabled = false;
    if (event.offsetX !== undefined) {
      this.lastX = event.offsetX;
      this.lastY = event.offsetY;
    } else {
      this.lastX = event.layerX - event.currentTarget.offsetLeft;
      this.lastY = event.layerY - event.currentTarget.offsetTop;
    }
    if (this.context)
      // begins new line
      this.context.beginPath();

    this.drawing = true;
  }

  drawEvent(event: any) {
    event.preventDefault();
    let currentX: number;
    let currentY: number;
    if (this.drawing) {
      // get current mouse position
      if (event.offsetX !== undefined) {
        currentX = event.offsetX;
        currentY = event.offsetY;
      } else {
        currentX = event.layerX - event.currentTarget.offsetLeft;
        currentY = event.layerY - event.currentTarget.offsetTop;
      }

      this.draw(this.lastX, this.lastY, currentX, currentY);

      // set current coordinates to last one
      this.lastX = currentX;
      this.lastY = currentY;
    }
  }

  draw(lX: number, lY: number, cX: number, cY: number) {
    if (this.context) {
      // line from
      this.context.moveTo(lX, lY);
      // to
      this.context.lineTo(cX, cY);
      // color
      this.context.strokeStyle = '#222';
      // draw it
      this.context.stroke();
    }
  }

  stopDrawing() {
    this.drawing = false;
  }

  clean() {
    this.dirty = false;
    if (this.context)
      this.context.clearRect(0, 0, this.canvas.width, this.canvas.height);
  }

  getDataURL(): string {
    if (this.canvas) return this.canvas.toDataURL('image/png', 1.0);
    else return '';
  }

  paintCanvas(firma: string): void {
    if (this.canvas) {
      this.dirty = true;
      let img = new Image();
      img.src = firma;
      let context = this.canvas.getContext('2d');
      img.onload = function () {
        if (context) {
          context.drawImage(img, 0, 0);
        }
      };
    }
  }

  @HostListener('mousedown', ['$event']) onMouseDown(event: any) {
    this.startDrawing(event);
  }
  @HostListener('touchstart', ['$event']) onTouchStart(event: any) {
    this.startDrawing(event);
  }
  @HostListener('pointerstart', ['$event']) onPinterStart(event: any) {
    this.startDrawing(event);
  }
  @HostListener('mousemove', ['$event']) onMouseMove(event: any) {
    this.drawEvent(event);
  }
  @HostListener('touchmove', ['$event']) onTouchMove(event: any) {
    this.drawEvent(event);
  }
  @HostListener('pointermove', ['$event']) onPointerMove(event: any) {
    this.drawEvent(event);
  }
  @HostListener('mouseup') onMouseUp() {
    this.stopDrawing();
  }
  @HostListener('mouseout') onMouseOut() {
    this.stopDrawing();
  }
  @HostListener('mouseleave') onMouseLeave() {
    this.stopDrawing();
  }
  @HostListener('touchend') onTouchEnd() {
    this.stopDrawing();
  }
  @HostListener('pointerend') onPointerEnd() {
    this.stopDrawing();
  }
  @HostListener('touchcancel') onTouchCancel() {
    this.stopDrawing();
  }
}
