import { AbstractControl, ValidationErrors, ValidatorFn } from '@angular/forms';

export const documentFormatValidator: ValidatorFn = (
  control: AbstractControl
): ValidationErrors | null => {
  const document: string = control.value.toUpperCase();
  if (!document || document === '') return null;
  let tpdocument = getTpDocument(document);
  if (tpdocument === 'NIF') return checkNIF(document);
  if (tpdocument === 'NIE') return checkNIE(document);
  if (tpdocument === 'CIF') return checkCIF(document);
  return { formatError: true };
};

export const documentRequiredValidator: ValidatorFn = (
  control: AbstractControl
): ValidationErrors | null => {
  const document = control.get('document');
  const passaport = control.get('passaport');
  return document &&
    passaport &&
    document.value === '' &&
    passaport.value === ''
    ? { documentRequired: true }
    : null;
};

export const documentDenunciantRequiredValidator: ValidatorFn = (
  control: AbstractControl
): ValidationErrors | null => {
  const enabledDenunciant = control.parent?.get('enableDenunciant');
  if (enabledDenunciant && !enabledDenunciant.value) return null;
  else return documentRequiredValidator(control);
};

function checkNIF(document: string): ValidationErrors | null {
  if (nifRegex.test(document)) {
    const endletter = letterFromDocument(document);
    if (document.charAt(document.length - 1) !== endletter) {
      return { badLetterError: true };
    } else return null;
  } else {
    return { nifFormatError: true };
  }
}

function checkNIE(document: string): ValidationErrors | null {
  //El format del NIE és: (X/Y/Z)9999999L
  if (nieRegex.test(document)) {
    const firsLetter = document.charAt(0);
    document =
      firsLetter === 'Y' ? '1' : (firsLetter === 'Z' ? '2' : '') + document;
    const endletter = letterFromDocument(document);
    if (document.charAt(document.length - 1) !== endletter) {
      return { badLetterError: true };
    } else return null;
  } else {
    return { nieFormatError: true };
  }
}

function checkCIF(document: string): ValidationErrors | null {
  //El format del CIF és: (A/B/E/G/F/H)99999999 o bé (N/P/Q/S/W)9999999L
  if (cifRegex.test(document) || newcifRegex.test(document)) {
    return null;
  } else {
    return { cifFormatError: true };
  }
}

export function getTpDocument(document: string): string {
  if (/^[0-9]/.test(document)) return 'NIF';
  if (/^[AEBGPQSFHNW]/.test(document)) return 'CIF';
  if (/^[XYZ]/.test(document)) return 'NIE';
  return '';
}

export function letterFromDocument(document: string) {
  const lletresCalcul = [
    'T',
    'R',
    'W',
    'A',
    'G',
    'M',
    'Y',
    'F',
    'P',
    'D',
    'X',
    'B',
    'N',
    'J',
    'Z',
    'S',
    'Q',
    'V',
    'H',
    'L',
    'C',
    'K',
    'E',
  ];
  let num = Number(plainDocument(document));
  return lletresCalcul[num % 23];
}

function plainDocument(document: string): string {
  let eliminaZeros: boolean = true;
  let newDocument: string = '';
  for (let i = 0; i < document.length; i++) {
    const currentLetter = document.charAt(i);
    if (
      /^[1-9]$/.test(currentLetter) ||
      (currentLetter === '0' && !eliminaZeros)
    ) {
      newDocument += currentLetter;
      eliminaZeros = false;
    }
  }
  return newDocument;
}

export const nifRegex = /^[0-9]{8}[TRWAGMYFPDXBNJZSQVHLCKE]$/i;
export const nieRegex = /^[XYZ][0-9]{7}[TRWAGMYFPDXBNJZSQVHLCKE]$/i;
export const cifRegex = /^[AEBGFHN][0-9]{8}[TRWAGMYFPDXBNJZSQVHLCKE]$/i;
export const newcifRegex = /^[NPQSW][0-9]{7}[A-Z]$/i;
