package rest.security;
import java.io.IOException;
import java.lang.reflect.AnnotatedElement;
import java.lang.reflect.Method;
import java.security.Principal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


import controller.security.TokenCtrl;
import controller.security.UsuariCtrl;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.Jws;
import jakarta.annotation.Priority;
import jakarta.ws.rs.Priorities;
import jakarta.ws.rs.container.ContainerRequestContext;
import jakarta.ws.rs.container.ContainerRequestFilter;
import jakarta.ws.rs.container.ResourceInfo;
import jakarta.ws.rs.core.Context;
import jakarta.ws.rs.core.HttpHeaders;
import jakarta.ws.rs.core.Response;
import jakarta.ws.rs.core.SecurityContext;
import jakarta.ws.rs.ext.Provider;
@Secured
@Provider
@Priority(Priorities.AUTHENTICATION)
public class AuthenticationFilter implements ContainerRequestFilter {

	private static final String REALM = "policia";
	private static final String AUTHENTICATION_SCHEME = "Bearer";

	//Inyección para recuperar los roles de llamada al método
	@Context
	private ResourceInfo resourceInfo;


	@Override
	public void filter(ContainerRequestContext requestContext) throws IOException {

		// Get the Authorization header from the request
		String authorizationHeader =
				requestContext.getHeaderString(HttpHeaders.AUTHORIZATION);

		// Validate the Authorization header
		if (!isTokenBasedAuthentication(authorizationHeader)) {
			abortWithForbidden(requestContext);
			return;
		}

		// Extract the token from the Authorization header
		String token = authorizationHeader
				.substring(AUTHENTICATION_SCHEME.length()).trim();

		try {

			// Validate the token
			TokenCtrl tokenCtrl=new TokenCtrl("");
			tokenCtrl.setAcces_token(token);
			Jws<Claims> jws=tokenCtrl.validateToken();
			String username=jws.getBody().getSubject();

			
			
			//Validación de permisos (ROLES)
			Method resourceMethod = resourceInfo.getResourceMethod();
			List<Option> methodOptions = extractOptions(resourceMethod);
			boolean ok=false;
			if (!methodOptions.isEmpty()) {
				ok=UsuariCtrl.isAuthorizedRequest(username, methodOptions.get(0),requestContext.getMethod());
			}
			else {
				ok=true;
			}
			if (ok) {
				makeSecurityContext(requestContext, username);
			//Set Principal of SecurityContext			
			}
			else {
				abortWithForbidden(requestContext);
			}

		} 
		catch (ExpiredJwtException exp) {
			abortWithExpired(requestContext);
		}
		catch (Exception e) {
			abortWithForbidden(requestContext);
		}
	}
	
	private void makeSecurityContext(ContainerRequestContext requestContext, String username) throws Exception {
		final SecurityContext currentSecurityContext = requestContext.getSecurityContext();
		requestContext.setSecurityContext(new SecurityContext() {

			@Override
			public Principal getUserPrincipal() {
				return () -> username;
			}

			@Override
			public boolean isUserInRole(String role) {
				try {
					return UsuariCtrl.userInRole(username, role);
				}
				catch (Exception exc) {
					return false;
				}
			}

			@Override
			public boolean isSecure() {
				return currentSecurityContext.isSecure();
			}

			@Override
			public String getAuthenticationScheme() {
				return AUTHENTICATION_SCHEME;
			}
		});   
		
	}

	//Extracci�n de ROLES inyectados
	private List<Option> extractOptions(AnnotatedElement annotatedElement) {
		if (annotatedElement == null) {
			return new ArrayList<Option>();
		} else {
			Secured secured = annotatedElement.getAnnotation(Secured.class);
			if (secured == null) {
				return new ArrayList<Option>();
			} else {
				Option[] options = secured.value();
				return Arrays.asList(options);
			}
		}
	}    

	private void checkPermissions(String userName, List<Option> allowedRoles, ContainerRequestContext requestContext) throws Exception {
		//Comprovaci�n de seguridad (usuario, role) seguridad Intranet CALVIA
/*		try {
			java.util.Vector<Usuari> v = wf.getDriver().retrieve(new Usuari(),"ENTRADA='"+userName.toUpperCase().trim()+"'"+" AND SWGRUP='N'",null);
			if (v==null || v.size()==0) {
				throw new Exception("Usuari no v�lid");
			} else if (v.size()>1) {
				throw new Exception("ERROR INTERN ! Varis usuaris/contrasenyes coincideixen !");
			}
			Usuari userIdentified=v.elementAt(0);
			wf.setDataObj(userIdentified);
			for (Role role : allowedRoles) { 
				String vrole[]=role.name().split("_");
				boolean authorized = userIdentified.isAuthorizedRequest(null, vrole[0], vrole[1], vrole[2], vrole[3]);
				if (!authorized) {
					abortWithUnauthorized(requestContext);
					return;
				}

			}
		}
		finally {
			if (con!=null) try {con.close();} catch (Exception exc) {}; 
		}*/

		// Check if the user contains one of the allowed roles
		// Throw an Exception if the user has not permission to execute the method
	}


	private boolean isTokenBasedAuthentication(String authorizationHeader) {

		// Check if the Authorization header is valid
		// It must not be null and must be prefixed with "Bearer" plus a whitespace
		// The authentication scheme comparison must be case-insensitive
		return authorizationHeader != null && authorizationHeader.toLowerCase()
				.startsWith(AUTHENTICATION_SCHEME.toLowerCase() + " ");
	}

	private void abortWithForbidden(ContainerRequestContext requestContext) {

		// Abort the filter chain with a 403 status code response
		// The WWW-Authenticate header is sent along with the response
		requestContext.abortWith(
				Response.status(Response.Status.FORBIDDEN)
				.header(HttpHeaders.WWW_AUTHENTICATE, 
						AUTHENTICATION_SCHEME + " realm=\"" + REALM + "\"").entity("Accés no autoritzat")
				.build());
	}

	private void abortWithExpired(ContainerRequestContext requestContext) {

		// Abort the filter chain with a 401 status code response
		// The WWW-Authenticate header is sent along with the response
		requestContext.abortWith(
				Response.status(Response.Status.UNAUTHORIZED)
				.header(HttpHeaders.WWW_AUTHENTICATE, 
						AUTHENTICATION_SCHEME + " realm=\"" + REALM + "\"").entity("Temps accés expirat")
				.build());
	}
	

}
